
:root {
    --main-color: #00824D;
    --secondary-color: #274247;
    --error-color: #DC3400;
}

.main-layout {
    gap: 20px;
}
.divider {
    margin-bottom: 10px;
}

.card {
    border: thin solid var(--main-color);
    margin: 20px;
    padding-top:10px;
    padding-bottom:20px;
    padding-left:20px;
    padding-right:20px;

}

table {
    width:100%;
}

td {

    white-space: nowrap;
overflow: hidden;
text-overflow: ellipsis;
}
table, th, td {
    border: 1px solid var(--main-color);
    padding-top:4px;
    padding-bottom:4px;
    padding-left:8px;
    padding-right:8px;
    border-collapse: collapse;
    text-align: right;
    font-size: 14px;
}

.small-header {
    color: var(--secondary-color);
    font-size: 24px
}

.row-container{
    display: flex;
}

@media (max-width: 1200px) {
    .row-container {
        flex-direction: column;
    }
    }

.graph-container {
    width: 100%;
    height: 100%;
    padding-top:20px;
    display: flex;
    flex-direction: column;
}

.graph-class {
    width: 100%;
    display: flex;
}
.table-container {
    display: flex;
    flex-direction: column;
}

.chart-row-container {
    display: flex;
}

.column {
    display: flex;
    width: 100%;
}


.dropdown-container {
    padding:20px;
}

.nav-container {
    display:flex;
    flex-direction: row;
    justify-content: space-between;
    border-bottom: thin solid var(--main-color);
    margin-bottom: 10px;
    padding:10px;
}

.nav-row-container {

    display:flex;
    flex-direction: row;
    align-items: flex-end;

}
.nav-item-container{
    height: fit-content;
    margin-right: 10px;
    padding-left: 10px;
    border-left: var(--main-color);
    border-left-width: 2px;
    border-left-style: solid;
}

.nav-item {
    color: var(--main-secondary);
    font-size:18px;

}

a.nav-item:hover {
    color: var(--main-color);
    font-size:18px
}
